/*!
\file atmega8_gps_logger_basismodul.c
\brief Hauptfile des Projekts ATmega8 GPS Logger
\author Martin Matysiak (mail@k621.de)
\author V. Pippan (webmaster@vpippan.at)
\date 2013-10-18
\version 20131018

Copyright (c) 2008 Martin Matysiak<br>
Copyright 2008-2013 V. Pippan (webmaster@vpippan.at)

This file is part of ATmega8 GPS Logger.

ATmega8 GPS Logger is free software: you can redistribute it and/or modify<br>
it under the terms of the GNU General Public License as published by<br>
the Free Software Foundation, either version 3 of the License, or any later version.<br>

ATmega8 GPS Logger is distributed in the hope that it will be useful,<br>
but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the<br>
GNU General Public License for more details.<br>

You should have received a copy of the GNU General Public License<br>
along with ATtiny2313 Cessna Lichtsteuerung.  If not, see<br>
<a href="http://www.gnu.org/licenses" target="blank">www.gnu.org/licenses</a>.

\todo
\li Code schöner schreiben, Kommtentare einfügen, Doxygen Doku verbessern
\li FAT, UART und MMC Libraries in eigenem Ordner speichern
\li In neuer Version auf FAT verzichten und Daten ohne Dateisystem auf SD Karte schreiben (siehe gLoggerMini) -> weniger Speicher für Code, schneller?
\li Wartezeit für GPS Datenempfang einstellbar machen (eventuell nur Positionsdaten auslesen?)
\li GPS Maus mit anderen Einstellungen initialisieren (höhere Datenrate, andere NMEA Sätze?)
\li Start / Stop aus dem File entfernen -> Überprüfen ob Datei vorhanden und dann neue anlegen (Dateinamen bei jedem starten ändern und damit neue Datei anlegen -> String append ?)
\li Akkuspannung messen und bei zu niedriger Spannung -> sleep mode (Datei abschließen), GPS und Zusatzmodul abschalten und warnen
\li GPS nur einschalten wenn Aufzeichnung aktiviert, sonst ausschalten
\li GPS Daten auf Display ausgeben, zusätzliche Sensoren, Jeti Kommunikation -> größerer uC benötigt? -> ATmega32?
\li Berechnungen für Oldtimerrallye Training machen und am Display ausgeben
\li Erkennung von Zusatzmodulen und Code für diese

*/

#include <avr/io.h>
#include <util/delay.h>
#include "gps.h"
#include "mmc.h"
#include "fat16.h"

//Definitionen ###########################################################################################

#define F_CPU 16000000L

#define LED_PORT PORTD
#define LED_STAT PD6
#define LED_WARN PD7

#define LEDCODE_OFF LED_PORT &= ~((1 << LED_WARN) | (1 << LED_STAT)) //0x00
#define LEDCODE_OK LED_PORT = (LED_PORT & ~(1 << LED_WARN)) | (1 << LED_STAT) //0x01
#define LEDCODE_WARNING LED_PORT = (LED_PORT & ~(1 << LED_STAT)) | (1 << LED_WARN) //0x02
#define LEDCODE_PROCESSING LED_PORT |= (1 << LED_WARN) | (1 << LED_STAT) //0x03
#define LEDCODE_BLINK LED_PORT ^= (1 << LED_STAT)
 
struct
{
	unsigned record:1;
	unsigned keydown:1;
} status;

/*! \brief Hauptfunktion
 *
 * Die Hauptfunktion enthält die Initialisierungen beim Programmstart und die Hauptschleife des
 * Programms.
 * */
int main (void)
{
 
	// Ports als Ausgänge definieren
	DDRD = DDRD | (1 << DDD6) | (1 << DDD7);

	LEDCODE_PROCESSING;

	// SD Karte + FAT initalisieren
	if ((mmc_init() != 0) || (InitFat16() != 0))
	{
		LEDCODE_WARNING;
		while(1);
	}

	LEDCODE_OK;

	// Log-Datei suchen und Position merken
	File logFile;

	// GPS Modul initalisieren
	gps_init();

	// Variablen initalisieren
	LEDCODE_OFF;
	status.record = 0;
	status.keydown = 0;
	char nmea_string[128]; //!< definiert ein character feld für nmea daten mit 128 zeichen länge

	//! Hauptschleife
	while(1)
	{
		// taster gedrückt oder taster war gedrückt
		if ((PIND & (1 << PIND2)) || (status.keydown == 1) )
		{
			LEDCODE_WARNING;
			
			// taster war im letzten durchlauf noch nicht gedrückt
			if (status.keydown == 0)
			{
				status.keydown = 1;
				_delay_ms(2000);
			}
			else // taster war im letzen durchlauf gedrückt
			{
				LEDCODE_PROCESSING;
				status.keydown = 0;
				
				// es wurde nicht aufgezeichnet
				if(status.record == 0)
				{
					fopen_((unsigned char*)"nmea.log",'a',&logFile);
					fputs_(&logFile,"***START***");
					status.record = 1;
				}
				else // es wurde aufgezeichnet
				{
					fputs_(&logFile,"***STOP***");
					fclose_(&logFile);
					status.record = 0;
				}
				
				_delay_ms(500);
				LEDCODE_OFF;
			}
		}

		// es soll aufgezeichnet werden
		if (status.record == 1)
		{
			gps_get_nmea(nmea_string, sizeof(nmea_string));
			fputs_(&logFile, nmea_string);
			LEDCODE_BLINK;
		}
		else // es soll nicht aufgezeichnet werden
		{
			_delay_ms(950);
			LEDCODE_WARNING;
			_delay_ms(50);
			LEDCODE_OFF;
		}
	}
}
